'   Copyright (1998)
'
'  File Name: rot2aa.bas
'  Revisions:
'             TCS Program Written
'  01/21/2003 TCS Updated element selection process
'
'  Written by tim@tshoemaker.com
'
'  This program lets the user select a cell or text element and
'  rotates it on the Z axis to the active angle.
'----------------------------------------------------------------------
' Use, modification, or distribution of this program implies that the
' user accepts all responsibility and liablility for the results.  No
' warranty of any kind is made by the author.  This program is free,
' please use it with that in mind.
'-----------------------------------------------------------------------

Declare Function LocateElement (Element as mbeElement, _
                                Prompt as String) as Long

'--------------------------------------------------------------------
'Subroutine that modifies an element
'--------------------------------------------------------------------
sub ModifyElement( elem as MbeElement )
    dim angle as double
    dim origin as MbePoint
    dim rMatrix(0 to 2, 0 to 2) as Double
    dim stat as long
    
    elem.changeAll = TRUE
    stat = elem.getRotation(rMatrix)
    
    rMatrix(0,1) = -rMatrix(0,1)
    rMatrix(0,2) = 0
    rMatrix(1,0) = -rMatrix(1,0)
    rMatrix(1,2) = 0
    rMatrix(2,0) = 0
    rMatrix(2,1) = 0
    rMatrix(2,2) = 1
    
    angle = MbeSettings.angle
    if elem.getOrigin(origin) = MBE_Success then
      if elem.rotate(rMatrix,origin) = MBE_Success then
        if elem.rotate(angle,0,0,origin) = MBE_Success then
          stat = elem.rewrite()
        end if
      end if
    end if  
end sub


'---------------------------------------------------------------
'
'   Main - main subroutine
'
'---------------------------------------------------------------
Sub Main
  Dim elem As New MbeElement
  Dim found As Long

  found = locateElement(elem, "Identify Text or Cell / Reset to Exit")
  do while found
    call ModifyElement( elem )
    found = locateElement(elem, "Identify Text or Cell / Reset to Exit")
  loop

End Sub


Function LocateElement (Element As mbeElement, Prompt As String) as Long

' LocateElement function (from LocateFn.bas file)
' enables location of individual graphic elements using MSbasic
' this function can be pasted into any macro as required
' Mapping with MicroStation, Volume 4: Macro Programming, chapter 4
' A.Bonnici, Nov 1997, geodata@ptbo.igs.net, www.ptbo.igs.net/~geodata
'
'  Revisions:
'  09/06/2002 TCS Revised to exit quitely if another command is picked
'  09/06/2002 TCS Revised to allow selection of any type element
'  01/21/2003 TCS Revised to allow selection of:  MBE_CellHeader
'                                                 MBE_SharedCell
'                                                 MBE_Text
'                                                 MBE_TextNode

   dim idPoint as mbePoint     'coordinates of user's data point
   dim filePos as long         'current position in design file
   dim elementFound as Long    'loop continuation flag
   dim continueScan as Long    'design file scanning status flag
   Dim correctType As Long     'added 01/21/03
   elementFound = FALSE        'set to continue loop
   continueScan = FALSE        'set to start scan at top of file
 

' STEP 1: START SCANNING LOOP

   Do While elementFound = FALSE
      mbeStartLocate 0, 0, 0, continueScan

' STEP 2: SEND DATA POINT

      If continueScan Then
         Call mbeSendDataPoint (idPoint)
      Else
         mbeWritePrompt Prompt
         mbeGetInput MBE_DataPointInput, MBE_ResetInput, _
                     MBE_CommandInput, MBE_KeyInInput

       Select Case MbeState.InputType
       Case MBE_DataPointInput
         stat = mbeState.getInputDataPoint (idPoint)
         mbeSendLastInput
       Case MBE_ResetInput
         End
       Case MBE_CommandInput, MBE_KeyInInput
         MbeSendLastInput
         End
       End Select

      End if   ' continue scanning

' STEP 3: CHECK FOR APPROPRIATE ELEMENT

      If mbeState.cmdResult <> MBE_AcceptQuery then
         stat = mbeMessageBox ("Element not found!", 2048)
         elementFound = FALSE   ' no element in vicinity
         continueScan = FALSE   ' start over

      Else
         filepos = Element.fromLocate ()

         'modified 01/23/03
         correctType = FALSE
         If Element.type = MBE_CellHeader or _
            Element.type = MBE_SharedCell or _
            Element.type = MBE_Text or _
            Element.type = MBE_TextNode Then correctType = TRUE

         If (correctType <> TRUE ) Then

            Msg$ = "Identified element is wrong type!"
            stat = mbeMessageBox (Msg, MBE_WarningIcon)
            elementFound = FALSE   ' wrong type
            continueScan = FALSE   ' start over
         'end modified 01/23/03

         Else
' STEP 4: GET USER CONFIRMATION

            Element.display MBE_Hilite
            mbeWritePrompt "Accept / Reject"
            mbeGetInput MBE_DataPointInput, MBE_ResetInput

            If mbeState.inputType = MBE_ResetInput Then
               Element.display MBE_NormalDraw
               elementFound = FALSE   ' user rejected
               continueScan = TRUE    ' continue search
            
            Else
               elementFound = TRUE    ' proper type and accepted
               LocateElement = TRUE

            End If   ' user acceptance
         End If   ' proper element type
      End If   ' element located
   Loop   ' while element not found

   mbeWritePrompt

end function
